# This file contains the terracotta functions for the tomcat charm.

add_jndi_terracotta_server () {
  juju-log "Adding terracotta server ${JUJU_REMOTE_UNIT} to jndi configuration"
  add_file_row "/etc/${VERSION}/${REMOTE_SERVICE}.jndi_tc" \
   "${JUJU_REMOTE_UNIT}" `relation-get private-address`:9510
}

add_terracotta_server () {
  juju-log "Adding terracotta server ${JUJU_REMOTE_UNIT} to configuration"
  add_file_row /etc/${VERSION}/terracotta.servers "${JUJU_REMOTE_UNIT}" \
   `relation-get private-address`:9510
}

check_terracotta () {
  # If the private address is not set the relation to terracotta is not ready.
  if [ -z "`relation-get private-address`" ]; then
    juju-log "Terracotta server ${JUJU_REMOTE_UNIT} not ready"
    exit 0
  fi
}

configure_jndi_terracotta () {
  # Get the path of the .d directory where the server.xml file is built.
  server_dir=`dotdee -d /etc/${VERSION}/server.xml`
  rm -f "${server_dir}"/12-terracotta-*
  # Get the path of the .d directory where the context.xml file is built.
  context_dir=`dotdee -d /etc/${VERSION}/context.xml`
  rm -f "${context_dir}"/15-terracotta-*
  for name in "${!jndi_terracotta_service[@]}"; do
    service=${jndi_terracotta_service[${name}]}
    if [ -n "${service}" ] && [ -e "/etc/${VERSION}/${service}.jndi_tc" ]; then
      jndi_name=`escape_name ${name}`
      (umask 0027; config_environment "${name}" `format_terracotta_servers "/etc/${VERSION}/${service}.jndi_tc"` java.lang.String > "${server_dir}/12-terracotta-${jndi_name}")
      config_resource_link "${name}" java.lang.String \
        > "${context_dir}/15-terracotta-${jndi_name}"
    fi
  done
  # The dotdee update function performs an alphanumeric concatenation of all
  # files in the managed collection in /etc/dotdee/etc/path/to/some.conf.d/*
  # to the build target file /etc/dotdee/etc/path/to/some.conf
  dotdee -u /etc/${VERSION}/server.xml
  dotdee -u /etc/${VERSION}/context.xml
}

config_terracotta_valve () {
  echo "  <Valve className=\"$1\" tcConfigUrl=\"$2\"/>"
}

configure_terracotta () {
  juju-log "Configuring terracotta"
  # Get the path of the .d directory where the context.xml file is built.
  dir=`dotdee -d /etc/${VERSION}/context.xml`
  if [ ${VERSION} != tomcat6 ]; then
    class=org.terracotta.session.TerracottaTomcat70xSessionValve
  else
    class=org.terracotta.session.TerracottaTomcat60xSessionValve
  fi
  config_terracotta_valve ${class} `format_terracotta_servers /etc/${VERSION}/terracotta.servers` \
    > "${dir}/45-terracotta"
  # Build the target file /etc/dotdee/etc/tomcatX/context.xml from context.xml.d
  dotdee -u /etc/${VERSION}/context.xml
}

create_terracotta_log_directory () {
  if [ ! -e /var/log/terracotta ]; then
    (umask 0027; mkdir /var/log/terracotta)
    chown ${VERSION}:adm /var/log/terracotta
  fi
}

format_terracotta_servers () {
  cut -f 2 "$1" | cut -d $'\n' -f 1- --output-delimiter=,
}

install_terracotta () {
  if [ ! -e /usr/local/lib/terracotta ]; then
    juju-log "Downloading terracotta..."
    url=`config-get terracotta_url`
    wget -nv -c -P "${TEMP}" "${url}"
    dist=`basename ${url}`
    check_checksum "${TEMP}/${dist}" `config-get terracotta_md5`

    juju-log "Installing terracotta..."
    tar -C /usr/local/lib -xzf "${TEMP}/${dist}"
    (cd /usr/local/lib; ln -s terracotta-* terracotta)
    (cd /usr/share/${VERSION}/lib; ln -s ../../../local/lib/terracotta/sessions/terracotta-session-*.jar)
    (cd /usr/share/${VERSION}/lib; ln -s ../../../local/lib/terracotta/common/terracotta-toolkit-*.jar)
  fi
}

parse_jndi_terracotta_config () {
  unset jndi_terracotta_service
  declare -Ag jndi_terracotta_service
  # Split the jndi_terracotta_config string on commas.
  IFS="," read -a tokens <<<"`config-get jndi_terracotta_config`"
  for token in "${tokens[@]}"; do
    # Split the token string using colon.
    IFS=":" read name service <<<"${token}"
    # Put the name and service in the array if they are not empty strings.
    if [ -n "${name}" ] && [ -n "${service}" ]; then
      jndi_terracotta_service[${name}]=${service}
    fi
  done
}

remove_jndi_terracotta () {
  juju-log "Removing terracotta jndi configuration"
  rm -f "/etc/${VERSION}/${REMOTE_SERVICE}.jndi_tc"
}

remove_jndi_terracotta_server () {
  juju-log "Removing terracotta server ${JUJU_REMOTE_UNIT} from jndi configuration"
  remove_file_row "/etc/${VERSION}/${REMOTE_SERVICE}.jndi_tc" "${JUJU_REMOTE_UNIT}"
}

remove_terracotta () {
  juju-log "Removing terracotta"
  rm -f /etc/${VERSION}/terracotta.servers
  # Get the path of the .d directory where the context.xml file is built.
  dir=`dotdee -d /etc/${VERSION}/context.xml`
  rm -f "${dir}/45-terracotta"
  # Build the target file /etc/dotdee/etc/tomcatX/context.xml from context.xml.d
  dotdee -u /etc/${VERSION}/context.xml
}

remove_terracotta_server () {
  juju-log "Removing terracotta server ${JUJU_REMOTE_UNIT} from configuration"
  remove_file_row /etc/${VERSION}/terracotta.servers "${JUJU_REMOTE_UNIT}"
}
